// src/services/errorHandler.js
import axios from 'axios';
import { jwtDecode } from 'jwt-decode';

export class ApiError extends Error {
    constructor(message, status, details = {}) {
      super(message);
      this.status = status;
      this.details = details;
    }
}
  
export const createApiClient = () => {
  const client = axios.create({
    baseURL: process.env.REACT_APP_API_URL,
    headers: {
      'X-Content-Type-Options': 'nosniff',
      'X-Frame-Options': 'DENY',
      'X-XSS-Protection': '1; mode=block'
    }
  });

  // Add request interceptor for token validation
  client.interceptors.request.use(async (config) => {
    const token = localStorage.getItem('token');
    if (!token) return config;

    try {
      const decodedToken = jwtDecode(token);
      const currentTime = Date.now() / 1000;

      if (decodedToken.exp < currentTime) {
        localStorage.removeItem('token');
        window.location.href = '/api/v1/login';
        throw new ApiError('Session expired', 401);
      }

      // Refresh token if expiring soon
      if (decodedToken.exp - currentTime < 300) {
        try {
          const response = await axios.post('/api/v1/refresh-token', { token });
          localStorage.setItem('token', response.data.token);
          config.headers.Authorization = `Bearer ${response.data.token}`;
        } catch (error) {
          localStorage.removeItem('token');
          window.location.href = '/api/v1/login';
          throw new ApiError('Failed to refresh token', 401);
        }
      }

      config.headers.Authorization = `Bearer ${token}`;
    } catch (error) {
      localStorage.removeItem('token');
      window.location.href = '/api/v1/login';
      throw new ApiError('Invalid token', 401);
    }

    // Add CSRF token if available
    const csrfToken = document.querySelector('meta[name="csrf-token"]')?.content;
    if (csrfToken) {
      config.headers['X-CSRF-Token'] = csrfToken;
    }

    return config;
  });

  // Enhanced response interceptor
  client.interceptors.response.use(
    response => response,
    error => {
      if (error.response?.status === 401) {
        localStorage.removeItem('token');
        window.location.href = '/api/v1/login';
        return Promise.reject(new ApiError('Session expired', 401));
      }

      // Handle rate limiting
      if (error.response?.status === 429) {
        return Promise.reject(new ApiError('Too many requests. Please try again later.', 429));
      }

      // Handle server errors
      if (error.response?.status >= 500) {
        return Promise.reject(new ApiError('Server error. Please try again later.', error.response.status));
      }
      
      return Promise.reject(new ApiError(
        error.response?.data?.message || 'An error occurred',
        error.response?.status,
        error.response?.data
      ));
    }
  );

  return client;
};